program MarioEdit;

uses Graphics, Crt, Defs, Procs;

type
    Item = record
      x, y, n: Integer;
    end;

var Data: Pointer;
    Level: array[0..99, 0..99] of Byte;
    Palette: array[0..1023] of Byte;
    NumObj, NumMonsters, NumPlats, Tex: Integer;
    ScrX, ScrY, SelX, SelY, SelTex: Integer;
    TexX, TexY, MarX, MarY, TexOffs: Integer;
    Key: Char;
    Edit, Content, Del: Boolean;
    LevelInfo: Info;

procedure LoadData(S: String);
  var f: File;
begin
  Assign(f, S);
  Reset(f, 1);
  GetMem(Data, DATA_SIZE);
  Seek(f, 54);
  BlockRead(f, Palette, 1024);
  BlockRead(f, Data^, DATA_SIZE);
  Close(f);
end;

procedure LoadLevel(S: String);
  var i, j: Integer;
      f: File;
      Rec: Item;
      M: MonsterRec;
begin
  for i := 0 to 99 do
  for j := 0 to 99 do
    Level[i, j] := 255;

  Assign(f, S);
  Reset(f, 1);

  BlockRead(f, LevelInfo, SizeOf(Info));

  for i := 1 to LevelInfo.NumTerrain do begin
    BlockRead(f, Rec, SizeOf(Rec));
    Level[Rec.x div 20, Rec.y div 14] := Rec.n;
  end;
  for i := 1 to LevelInfo.NumMonsters do begin
    BlockRead(f, M, SizeOf(M));
    case M.n of
      0: Level[M.x div 20, M.y div 14] := 61;
      1: Level[M.x div 20, M.y div 14] := 70;
      2: Level[M.x div 20, M.y div 14] := 72;
      3: Level[M.x div 20, M.y div 14] := 76;
    end;
  end;
  BlockRead(f, Rec, SizeOf(Rec));
  MarX := Rec.x div 20;
  MarY := Rec.y div 14;

  Close(f);
end;

procedure SaveLevel(S: String);
  var i, j: Integer;
      f: File;
      Rec: Item;
begin
  NumObj := 0;

  LevelInfo.NumTerrain := 0;
  LevelInfo.NumMonsters := 0;

  for i := 0 to 99 do
  for j := 0 to 99 do begin
    if (Level[i, j] <> 255) and ((Level[i, j] < 61) or (Level[i, j] > 78)) then Inc(LevelInfo.NumTerrain);
    if (Level[i, j] > 60) and (Level[i, j] < 79) then Inc(LevelInfo.NumMonsters);
  end;

  Assign(f, S);
  Rewrite(f, 1);

  if ParamCount <> 0 then
    Val(ParamStr(1), LevelInfo.NumTex, ScrX);

  BlockWrite(f, LevelInfo, SizeOf(Info));

  for i := 0 to 99 do
  for j := 0 to 99 do begin
    if (Level[i, j] <> 255) and ((Level[i, j] < 61) or (Level[i, j] > 78)) then begin
      Rec.x := i * 20;
      Rec.y := j * 14;
      Rec.n := Level[i, j];
      BlockWrite(f, Rec, SizeOf(Rec));
    end;
  end;
  for i := 0 to 99 do
  for j := 0 to 99 do begin
    if (Level[i, j] > 60) and (Level[i, j] < 79) then begin
      Rec.x := i * 20;
      Rec.y := j * 14;
      case Level[i, j] of
        61: Rec.n := 0;
        70, 71: Rec.n := 1;
        72, 73: Rec.n := 2;
        76: Rec.n := 3;
      end;
      BlockWrite(f, Rec, SizeOf(Rec));
    end;
  end;
  Rec.x := MarX * 20;
  Rec.y := MarY * 14;
  Rec.n := 255;
  BlockWrite(f, Rec, SizeOf(Rec));

  Close(f);
end;

procedure SetPal;
  var i: Integer;
      C: Byte;
begin
  for i := 1 to 255 do
    SetRGB(i, Palette[i * 4 + 2], Palette[i * 4 + 1], Palette[i * 4]);
  for i := 200 to 216 do begin
    C := (i - 200) * 4;
    SetRGB(i, C, C, C);
    C := 64 - (i - 200) * 4;
    SetRGB(i + 16, C, C, C);
  end;
  SetRGB(255, 255, 255, 255);
end;

procedure DrawWindow;
  var i, j: Integer;
begin
  Bar_(2, 142, 316, 56, 210);
  HorLine_(0, 140, 319, 216);
  HorLine_(1, 141, 317, 216);
  VertLine_(0, 140, 60, 216);
  VertLine_(1, 141, 59, 216);

  HorLine_(2, 198, 317, 206);
  HorLine_(1, 199, 319, 206);
  VertLine_(318, 141, 60, 206);
  VertLine_(319, 140, 58, 206);

  for i := 0 to 9 do
  for j := 0 to 2 do
    PutImage(i * 20 + 110, j * 14 + 150, 20, 14, 0, ((i + TexOffs) * 3 + j) * 14, 20, Data);

  OutChar(6, 146, 'M', 0);
  OutChar(5, 145, 'M', 255);
  OutString(16, 146, '-Mario', 0);
  OutString(15, 145, '-Mario', 47);

  Rect_(107, 147, 205, 47, 47);
end;

procedure RefreshScreen;
  var i, j, x, y: Integer;
      Color: Byte;
begin
  Cls(0);
  Color := (ScrX - ((ScrX div 66) * 66)) div 2 + 200;
  for i := 0 to 319 do begin
    VertLine_(i, 0, 140, Color);
    Inc(Color);
    if Color > 232 then Color := 200;
  end;
  for i := 0 to 99 do
  for j := 0 to 99 do begin
    if Level[i, j] <> 200 then begin
      if (i * 20 >= ScrX - 20) and (j * 14 >= ScrY - 14) then
      if (i * 20 < ScrX + 320) and (j * 14 < ScrY + 140) then
        if Level[i, j] <> 255 then
          PutImage(i * 20 - ScrX, j * 14 - ScrY, 20, 14, 0, Level[i, j] * 14, 20, Data);
    end;
  end;
  OutChar(MarX * 20 - ScrX + 6, MarY * 14 - ScrY + 3, 'M', 255);
  DrawWindow;
  if Edit then begin
    PutImage(SelX * 20, SelY * 14, 20, 14, 0, SelTex * 14, 20, Data);
    Rect_(SelX * 20, SelY * 14, 19, 13, 255);
  end
  else
    Rect_(TexX * 20 + 110, TexY * 14 + 150, 19, 13, 255);
  if Content then begin
    OutString(11, 121, 'Fill mode on', 0);
    OutString(10, 120, 'Fill mode on', RED_COLOR);
  end;
  if Del then begin
    OutString(11, 121, 'Delete mode on', 0);
    OutString(10, 120, 'Delete mode on', RED_COLOR);
  end;
  VertRet;
  CopyToScreen(ScreenBuf);
end;

begin
  if ParamCount < 1 then Exit;
  SelX := 0;
  SelY := 0;
  TexX := 0;
  TexY := 0;
  TexOffs := 0;
  Edit := True;
  Content := False;
  SelTex := 0;
  SetMode($13);
  SetScreenBuf(SetBuf);
  LoadData(DATA_FILE);
  LoadFont(14, 'mario.fnt');
  SetPal;
  LoadLevel(ParamStr(1));
  ScrX := MarX * 20;
  ScrY := MarY * 14;
  if ScrX > 84 * 20 then ScrX := 84 * 20;
  if ScrY > 90 * 14 then ScrY := 90 * 14;

  RefreshScreen;
  repeat
    Key := ReadKey;

    case Key of
      'f': begin Content := not Content; Del := False; end;
      'd': begin Del := not Del; Content := False; end;
      'm': begin
             MarX := ScrX div 20 + SelX;
             MarY := ScrY div 14 + SelY;
           end;
      #77: if Edit then Inc(SelX) else Inc(TexX);
      #75: if Edit then Dec(SelX) else Dec(TexX);
      #80: if Edit then Inc(SelY) else Inc(TexY);
      #72: if Edit then Dec(SelY) else Dec(TexY);
      #9:  Edit := not Edit;
      #13: if not Edit then
             SelTex := (TexX + TexOffs) * 3 + TexY
           else
             Level[ScrX div 20 + SelX, ScrY div 14 + SelY] := SelTex;
      #83: Level[ScrX div 20 + SelX, ScrY div 14 + SelY] := 255;
    end;
    if Content then Level[ScrX div 20 + SelX, ScrY div 14 + SelY] := SelTex;
    if Del then Level[ScrX div 20 + SelX, ScrY div 14 + SelY] := 255;

    if SelX > 15 then begin
      SelX := 15;
      ScrX := ScrX + 20;
    end;
    if SelX < 0 then begin
      SelX := 0;
      ScrX := ScrX - 20;
    end;
    if SelY > 9 then begin
      SelY := 9;
      ScrY := ScrY + 14;
    end;
    if SelY < 0 then begin
      SelY := 0;
      ScrY := ScrY - 14;
    end;

    if TexX < 0 then begin
      TexX := 0;
      Dec(TexOffs);
      if TexOffs < 0 then TexOffs := 0;
    end;
    if TexX > 9 then begin
      TexX := 9;
      Inc(TexOffs);
      if TexOffs > 23 then TexOffs := 23;
    end;
    if TexY < 0 then TexY := 0;
    if TexY > 2 then TexY := 2;

    if ScrX < 0 then ScrX := 0;
    if ScrY < 0 then ScrY := 0;
    if ScrX > 84 * 20 then ScrX := 84 * 20;
    if ScrY > 90 * 14 then ScrY := 90 * 14;

    RefreshScreen;

    asm
      mov ax, 0C00h
      int 21h
    end;
  until Key = #27;
  SetMode(3);
  SaveLevel(ParamStr(1));
end.