unit Player;

interface

uses Graphics, Defs, Monsters, Procs, SB;

type
    PMario = ^TMario;
    TMario = object
      MarioX, MarioY: Integer;
      Gold, Lives: Integer;
      Frame, FrameTime, Speed, JumpTime: Integer;
      Right, Left, InAir, AltJump, A1, A2: Boolean;
      DeadFlag: Boolean;
      constructor Init;
      destructor Done;
      procedure SetCommonParams;
      procedure PutInLevel(x, y: Integer);
      procedure HandleEvent(KeyCode: Byte; Ally: PMonster);
      procedure Draw(x, y: Integer);
      function Move(dx, dy: Integer): Boolean;
    end;

implementation

constructor TMario.Init;
begin
  Lives := MAX_LIVES;
  SetCommonParams;
  A1 := True;
  A2 := True;
end;

destructor TMario.Done;
begin
end;

procedure TMario.SetCommonParams;
begin
  Gold := 0;
  Frame := 0;
  FrameTime := MARIO_FRAME_SPEED;
  Right := False;
  Left := False;
  InAir := False;
  Speed := MARIO_SPEED;
  JumpTime := 0;
  AltJump := False;
end;

procedure TMario.PutInLevel(x, y: Integer);
begin
  MarioX := x;
  MarioY := y;
  SetCommonParams;
end;

procedure TMario.HandleEvent(KeyCode: Byte; Ally: PMonster);
  var Temp: Boolean;
begin
  A2 := A1;
  case KeyCode of
    29: Speed := MARIO_MAX_SPEED;
    157: Speed := MARIO_SPEED;
    205: Right := False;
    203: Left := False;
    77: begin Right := True; Left := False; A1 := True; end;
    75: begin Left := True; Right := False; A1 := False; end;
    56: if not InAir and not AltJump then begin
          JumpTime := MARIO_JUMP_TIME;
          AltJump := True;
   {       ResetDSP;
          Playback(Sound, 3810, 11025);}
        end;
    184: begin
           JumpTime := 0;
           AltJump := False;
         end;
  end;

  if Right then Temp := Move(Speed, 0);
  if Left then Temp := Move(-Speed, 0);

  if not Right and not Left then Frame := 0;

  if JumpTime <> 0 then begin
    InAir := Move(0, -2);
    Frame := 3;
  end
  else begin
    InAir := Move(0, 2);
    if InAir then Frame := 2;
  end;

  if MarioX < 0 then MarioX := 0;
  if MarioY < 0 then MarioY := 0;
  if MarioX > LEVEL_WIDTH - 20 then MarioX := LEVEL_WIDTH - 20;
  if MarioY > LEVEL_HEIGHT + 28 then begin
    MarioY := LEVEL_HEIGHT + 28;
    DeadFlag := True;
    if not FindParam(PASSWORD_3) then Dec(Lives);
  end
  else DeadFlag := False;

  Dec(JumpTime);
  if JumpTime < 0 then JumpTime := 0;

  if not InAir and (Left or Right) then begin
    Dec(FrameTime);
    if FrameTime < 0 then begin
      FrameTime := MARIO_FRAME_SPEED;
      if Frame = 0 then
        Frame := 1
      else
        Frame :=0;
    end;
    if not Temp then Frame := 0;
  end;
end;

procedure TMario.Draw(x, y: Integer);
begin
  if A1 <> A2 then Flip__H(20, 112, MARIO_SPRITE * 20 * 14, Data);
  PutImage(MarioX - x, MarioY - y - 14, 20, 28, 0, (Frame * 2 + MARIO_SPRITE) * 14, 20, Data);
end;

function TMario.Move(dx, dy: Integer): Boolean;
  var x, y, n, i, j: Integer;
      Allow: Boolean;
begin
  x := MarioX + dx;
  y := MarioY + dy;
  Allow := True;

  for i := (x - dx) div 20 - 1 to (x - dx) div 20 + 1 do
  for j := (y - dx) div 14 - 1 to (y - dy) div 14 + 1 do begin
    if (x > i * 20 - 18) and (x < i * 20 + 18) then
    if (y > j * 14 - 14) and (y < j * 14 + 18) then begin
      n := Level^[i + 1, j + 1];
      if (n <> 255) and Allow then
        case SOLID_TABLE[n + 1] of
          0: Allow := True;
          1: Allow := False;
          2: if (dx = 0) and (dy < 0) then Allow := True else Allow := False;
        end;
      if n = MONEY_SPRITE then begin
        Level^[i + 1, j + 1] := WIZARD_BEG;
        Inc(Gold);
      end;
    end;
  end;

  if Allow then begin
    MarioX := MarioX + dx;
    MarioY := MarioY + dy;
  end
  else if dx = 0 then
    JumpTime := 0;

  Move := Allow;
end;

end.
